@@ -2,22 +2,20 @@
 """Example for aiohttp.web websocket server
 """
 
-import asyncio
 import os
 
-from aiohttp.web import (Application, Response, WebSocketResponse, WSMsgType,
-                         run_app)
+from aiohttp import web
 
 
 WS_FILE = os.path.join(os.path.dirname(__file__), 'websocket.html')
 
 
 async def wshandler(request):
-    resp = WebSocketResponse()
-    ok, protocol = resp.can_prepare(request)
-    if not ok:
+    resp = web.WebSocketResponse()
+    available = resp.can_prepare(request)
+    if not available:
         with open(WS_FILE, 'rb') as fp:
-            return Response(body=fp.read(), content_type='text/html')
+            return web.Response(body=fp.read(), content_type='text/html')
 
     await resp.prepare(request)
 
@@ -28,7 +26,7 @@ async def wshandler(request):
         request.app['sockets'].append(resp)
 
         async for msg in resp:
-            if msg.type == WSMsgType.TEXT:
+            if msg.type == web.WSMsgType.TEXT:
                 for ws in request.app['sockets']:
                     if ws is not resp:
                         await ws.send_str(msg.data)
@@ -48,14 +46,12 @@ async def on_shutdown(app):
         await ws.close()
 
 
-async def init(loop):
-    app = Application()
+def init():
+    app = web.Application()
     app['sockets'] = []
     app.router.add_get('/', wshandler)
     app.on_shutdown.append(on_shutdown)
     return app
 
 
-loop = asyncio.get_event_loop()
-app = loop.run_until_complete(init(loop))
-run_app(app)
+web.run_app(init())
